;
;       PIC12C508 Macro vision remover chip ver 1.0
;       Written by the andrewm 021198.
;
;       Revision History:
;
;       Ver 1.0 Uses no VSync timmer
;
;       Uses Microchip assembler mnemonics and the
;       Microchip MPASM assembler.  
;
;                       _______  _______
;                      |       \/       |
;                Vdd --+ 1            8 +-- Vss
;                      |                |
;        Visible Out --+ 2 GP5        7 +-- 
;                      |                |
;           Burst in --+ 3 GP4    GP1 6 +-- Lower Cut
;                      |                |
;           Vsync in --+ 4 GP3    GP2 5 +-- Upper Cut
;                      |                |
;                      +----------------+
;
	list    p=12c508
	radix   HEX
	include "p12c508.inc"

	__FUSES _MCLRE_OFF & _CP_OFF & _WDT_OFF & _IntRC_OSC

	cblock  0x07    ;Leave 7 bytes for the control registers

		LINE	;Line counter / 2
		ODD	;Even/Odd toggle for Line/2 divider
		TimeBak	;Back up of TMR0 so TICK doesnt occur 1/2 way through test
		TMR1	;High byte of TMR0 in software
		OvTest  ;Over flow test for counter
		Delay	;Counter for delay routine
	endc

	org     0x00            ;Start of code space 
	
	movwf   OSCCAL          ;Move W into OSCCAL (this is not nesc but hey)


start   movlw   b'11000100'     ;1 = disable wake up on pin change
                                ;1 = disable weak pullups on GP0,1,3
                                ;0 = Timmer0 is free running counter (not pulse accu)
                                ;0 = Timmer0 is clocked leading edge
                                ;0 = prescaler assinged to timmer0 not WDT
                                ;100 = Prescaler divide ratio = 32
	option			;This command moves W (11000110 from above) to OPTION register


; At the start of a new line TRIS should make sure that 
;
; GP0 = Input	(not used)
; GP1 = Output	(Lower_Cut/Cut_on_burst enable) 
; GP2 = Output	(Upper_Cut/Cut_on_visible enable)
; GP3 = Input	(VSync in)
; GP4 = Input   (Burst in)
; GP5 = Output  (Visible Out. this is only an output during a Visible ie. Tristated_wired_AND)
;
; Also that
;
; GP1 = LOW	(Lower_Cut = OFF)
; GP2 = LOW	(Upper_Cut = OFF)
; GP5 = LOW	(Pull_down VISIBLE when output active)
;


NewLine movlw   b'11011001'     ;Make GP1 and GP2 Outputs rest inputs
	tris    GPIO

	bcf	GPIO,1		;Clear bit 1 in GPIO to make Pin 6 low (disable lower cut)
	bcf	GPIO,2		;Clear bit 2 in GPIO to make Pin 5 low (disable upper cut)
	bcf	GPIO,5		;Clear bit 5 in GPIO to make Pin 2 low (for pull down Visible)
	clrf	LINE		;Reset the line/2 counter
	clrf	TMR1		;Reset the high byte of the 16 bit counter
	bcf	ODD,0		;Set the ODD flag to off
	clrf	TMR0		;Clear the free running counter
	clrf	OvTest		;Clear the OverFlow test counter
;
; Here we wait for a BURST signal to indicate the start of a new line.
;
; After each burst signal we incriment the Line/2 counter by half
;
; After 3 lines (0x01 for the Line/2 counter then we jump out of this first section
; as the upper macro vision area has started
;
; At the start of the picture there are several false Burst signals before VSycn is over
; So this first part of counting should see if VSync is still active.
;
; If it is then the real picture has no started and we should go back to the "NewLine"
;

WaiVis1	btfsc	GPIO,4		;If GP4 is not low then wait here for start of BURST
	goto	WaiVis1
	btfsc	ODD,0		;If the flag is set the do next instuction
	incf	LINE,F		;so that incriment only happens every second line
	comf	ODD,F		;invert ODD toggle
	
	movlw	h'01'		;Load 0x02 into W
	xorwf	LINE,W		;XOR LINE with 0x02 leaving result in W
	btfsc	STATUS,Z	;If the result is ZERO then jump to UpStrt
	goto	UpStrt

ChkVsy1
	btfss	GPIO,3		;Check for VSync and reset line counter if there
	goto	NewLine		;is one
	goto	WaiVis1		;other wise just wait for next BURST



UpStrt	bsf	GPIO,2		;Enable Upper Cut


WaiVis2	btfsc	GPIO,4		;If pin 4 is not low then wait here for start of BURST
	goto	WaiVis2


	movlw	h'0C'		;delay for 0E x 3 instruction
	movwf	Delay
DlyLop1	decfsz	Delay,1
	goto	DlyLop1	

brk	bcf	GPIO,5		;Make sure GPIO5 is low
	movlw   b'11011001'     ;Make GP1 and GP2 ***AND*** GP5 Outputs 
	tris    GPIO

	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop

	btfsc	ODD,0		;If the flag is set the do next instuction
	incf	LINE,F		;so that incriment only happens every second line
	comf	ODD,F		;invert ODD toggle
	
	movlw	h'08'		;Load 0x08 into W
	xorwf	LINE,W		;XOR LINE with 0x07 leaving result in W
	btfsc	STATUS,Z	;If the result is ZERO then jump to UpEnd
	goto	UpEnd

	movlw   b'11111001'     ;Make GP1 and GP2 and disable GP5 Output 
	tris    GPIO

	goto	WaiVis2	

UpEnd	bcf	GPIO,2		;Disable Upper Cut
	movlw   b'11111001'     ;Disable GP5 Output 
	tris    GPIO



WaiVis3	btfsc	GPIO,4		;If pin 4 is not low then wait here for start of BURST
	goto	WaiVis3
	btfsc	ODD,0		;If the flag is set the do next instuction
	incf	LINE,F		;so that incriment only happens every second line
	comf	ODD,F		;invert ODD toggle
	
	movlw	h'93'		;Load 0x93 into W
	xorwf	LINE,W		;XOR LINE with 0x93 leaving result in W
	btfsc	STATUS,Z	;If the result is ZERO then jump to LowStrt
	goto	LowStrt

ChkVsyn	movf	TMR0,W		;Get value in timmer
	movwf	TimeBak		;Save it in working register
	btfss	OvTest,0	;Test the overflow test flag
	goto	NoOFlow		;If it is clear there cant be an overflow
	btfsc	TimeBak,7	;test MSB of TMR0 if it is low given OvTest is high
	goto	NoOFlow		;an overflow has occoured

	incf	TMR1,F		;and incriment TMR1
	bcf	OvTest,0	;Make OvTest follow TRM0[7]
	goto	SkipNo

NoOFlow	btfsc	TimeBak,7	;Test MSB of TMR0 backup if it is high then make
	bsf	OvTest,0	;OvTest  follow it

SkipNo
	movlw	h'02'		;Test TMR1 (software controlled high byte of timmer)
	subwf	TMR1,W
	btfss	STATUS,C
        goto    WaiVis3
	movlw	h'60'		;if TMR0 is greater than 12e
	subwf	TMR0,W		;then there is a possible vsync
	btfss	STATUS,C
	goto	WaiVis3		;nope go back and wait for next line

	bcf	GPIO,1		;Turn off lower cut
	

	movlw	h'60'		;No matter how long we wait make sure TMRO does not wrap
	movwf	TMR0

	btfss	GPIO,3		;The actual check for vsync
	goto	NewLine	

	goto	WaiVis3	

LowStrt	bsf	GPIO,1		;Enable Lower Cut

	goto	ChkVsyn


	end


